program inicioSesion;

{$APPTYPE CONSOLE}

uses
  SysUtils, inifiles, Dialogs, windows, Classes, shellapi, Controls;


function obtenerDirectorioWindows : TFileName;
var
  WinDir: array [0..MAX_PATH-1] of char;
begin
  SetString(Result, WinDir, GetWindowsDirectory(WinDir, MAX_PATH));
end;


function obtenerDirectorioSistema : TFileName;
var
  SysDir: array [0..MAX_PATH-1] of char;
begin
  SetString(Result, SysDir, GetSystemDirectory(SysDir, MAX_PATH));
end;


function ejecutarEsperar (aplicacion: String; parametros : string): Cardinal;
var
  StartInfo: TStartupInfo;
  ProcInfo: TProcessInformation;

begin
  Result := 0;

  FillChar(StartInfo, SizeOf(StartInfo), 0);
  StartInfo.cb := SizeOf(StartInfo);

  if CreateProcess(
    PChar(aplicacion), PChar(parametros), nil, nil, false, 0,
    nil, nil, StartInfo, ProcInfo)
  then
    Result := ProcInfo.hProcess;
end;


function ejecutarFichero (fichero, parametros : string) : boolean;
var
  resEj : THandle;
begin
  resEj := 0;
  resEj := ShellExecute(resEj, 'open', PChar(fichero),
        PChar(parametros), nil, SW_SHOWNORMAL);
  if resEj < 32 then
  begin
    result := false;
  end
  else
  begin
    result := true;
  end;
end;


function obtenerVariableEntorno (variable : string) : String;
var
  Env : PChar;
  valorVariable : string;
begin
  Result := '';
  Env := GetEnvironmentStrings;
  While Env^ <> #0 do
  begin
    valorVariable := StrPas(Env);
    if Pos (variable, valorVariable) = 1 then
    begin
      Result := copy (valorVariable, length(variable) + 1,
          length(valorVariable));
      exit;
    end;
    Inc(Env, StrLen(Env) + 1);
  end;
end;


procedure cerrarSesion;
var
  rl, flgs : Cardinal;
  hToken : Cardinal;
  tkp : TOKEN_PRIVILEGES;
begin
  flgs := 0;
  flgs := (flgs and (not (EWX_REBOOT or EWX_SHUTDOWN or EWX_POWEROFF))) or EWX_LOGOFF;
  if Win32Platform = VER_PLATFORM_WIN32_NT then
  begin
    if not OpenProcessToken(GetCurrentProcess, TOKEN_ADJUST_PRIVILEGES
             or TOKEN_QUERY, hToken) then
      //silenciosa
    else
    begin
      if LookupPrivilegeValue(nil, 'SeShutdownPrivilege', tkp.Privileges[0].Luid) then
      begin
        tkp.Privileges[0].Attributes := SE_PRIVILEGE_ENABLED;
        tkp.PrivilegeCount           := 1;
        AdjustTokenPrivileges(hToken, False, tkp, 0, nil, rl);
        if GetLastError <> ERROR_SUCCESS then
          Writeln('Se ha producido un error al intentar ajustar los privilegios.');
      end;
    end;
  end;
  //     else begin
  ExitWindowsEx(flgs, 0);
  //     end;
end;


var
  rutaPerfil : string;
  ficheroOrigen : string;
  ficheroDestino : string;
  nombreServidor : string;
  nombreUsuario : string;
  nombreEquipoCliente : string;
  numConexiones : integer;
  avisarUsuario : Boolean;
  nombreServidorPref : string;
  mensaje : string;
  permitirInicio : Boolean;
  cerrarSesionUsuario : Boolean;
  iniciarSTS : Boolean;
  controlRemoto : Boolean;
  usuAdministrador : Boolean;
  usuControlRemoto : string;
  listaFichero : TStringList;
  idSesion : string;
  numProceso : Cardinal;
const
  docInfo = '\\servidor\configuracion\info.ini';

begin
  try
    //comprobamos si le corresponde iniciar sesin en este servidor
    nombreServidor := obtenerVariableEntorno ('COMPUTERNAME=');
    nombreUsuario :=  obtenerVariableEntorno ('USERNAME=');
    with tinifile.create (docInfo) do
    try
      nombreServidorPref := ReadString (nombreUsuario, 'Servidor preferido', '');
      avisarUsuario :=  readBool (nombreUsuario, 'Mostrar aviso', false);
      permitirInicio := readBool (nombreUsuario, 'Permitir inicio sesin', true);
      cerrarSesionUsuario := readBool (nombreUsuario, 'Cerrar sesin', false);
    finally
      free;
    end;
    if not permitirInicio then
    begin
      if avisarUsuario then
        MessageDlg('Atencin: no tiene permiso para inciar sesin. Consulte ' +
            'con el departamento de Informtica. ' +
            'Pulse Aceptar para cerrar la sesin.', mtWarning, [mbok], 0);
      cerrarSesion;
    end;
    if avisarUsuario and (AnsiUpperCase (nombreServidor) <>
        AnsiUpperCase (nombreServidorPref)) then
    begin
      if cerrarSesionUsuario then
        mensaje := 'Atencin: su servidor por defecto es [' +
            nombreServidorPref + ']. Se ha conectado a [' +
            nombreServidor + '].' + chr(13) + chr(13) + chr(13) +
            'Pulse Aceptar para cerrar la sesin.'
      else
        mensaje := 'Atencin: su servidor por defecto es [' +
            nombreServidorPref + ']. Se ha conectado a [' +
            nombreServidor + '].';
      MessageDlg(mensaje, mtWarning, [mbok], 0);
      if cerrarSesionUsuario then
      begin
        cerrarSesion;
      end;
    end;
    //***** FIN comprobamos si le corresponde iniciar sesin en este servidor

    rutaPerfil := obtenerVariableEntorno ('USERPROFILE=');

    //**** PARA PROGRAMA DE CONTABILIDAD ****

    //si no existe "Mis documentos" lo creamos
    if not DirectoryExists (rutaPerfil + '\Mis documentos') then
      MkDir(rutaPerfil + '\Mis documentos');

    //si no existe "TEMPCONT" lo creamos
    if not DirectoryExists (rutaPerfil + '\Mis documentos\TEMPCONT') then
      MkDir(rutaPerfil + '\Mis documentos\TEMPCONT');

    //escribimos en apptao.ini del usuario el valor de la carpeta anterior
    //para que guarde los documentos temporales de CONTABILIDAD en esta ruta
    with tinifile.create (rutaPerfil + '\WINDOWS\CONTABILIDAD.INI') do
    try
      writeString ('Documentos', 'Temporales',
          rutaPerfil + '\Mis documentos\TEMPCONT');
      writeString ('Inicio', 'Activo', '0');
    finally
      free;
    end;

    //Crear directorio "plantilla" si no existe (para normal.dot)
    if not DirectoryExists (rutaPerfil + '\plantilla') then
      MkDir(rutaPerfil + '\plantilla');
    //**** FIN PARA PROGRAMA DE CONTABILIDAD ****



    //**** PARA GESTIN ****

    ficheroOrigen := 'c:\gestion\gestion.ini';
    ficheroDestino := rutaPerfil + '\WINDOWS\gestion.ini';
    if FileExists(ficheroOrigen) then
      CopyFile (pchar(ficheroOrigen), pchar(ficheroDestino), false);

    //**** FIN PARA GESTIN ****


    //*******PARA GUARDAR EN QU SERVIDOR EST CADA USUARIO *******
    nombreEquipoCliente :=  obtenerVariableEntorno ('CLIENTNAME=');
    //escribimos en info.ini los datos del usuario
    with tinifile.create (docInfo) do
    try
      numConexiones := ReadInteger (nombreUsuario, 'N Conexiones', 0);
      numConexiones := numConexiones + 1;
      writeString (nombreUsuario, 'Servidor', nombreServidor);
      writeString (nombreUsuario, 'Equipo cliente', nombreEquipoCliente);
      writeString (nombreUsuario, 'Hora Inicio sesion', datetimetostr(now));
      WriteInteger (nombreUsuario, 'N Conexiones', numConexiones);
    finally
      free;
    end;
    //*******FIN PARA GUARDAR EN QU SERVIDOR EST CADA USUARIO *******


    //Iniciar Administrador de servicios de Terminal Server
    //y hacer control remoto si as se ha indicado
    with tinifile.create (docInfo) do
    try
      iniciarSTS := ReadBool (nombreUsuario, 'Iniciar servicios TS', false);
      controlRemoto := ReadBool (nombreUsuario, 'Control remoto', false);
      usuAdministrador := ReadBool (nombreUsuario, 'Administrador', false);
      usuControlRemoto := ReadString (nombreUsuario, 'Usuario control remoto', '');
    finally
      free;
    end;

    if (usuAdministrador = true) and (usuControlRemoto <> '')
        and (controlRemoto = true) then
    begin
      with tinifile.create (docInfo) do
      try
        WriteString (nombreUsuario, 'Usuario control remoto', '');
      finally
        free;
      end;

      //ejecutarFichero ('cmd.exe', '/c query user ' +
      //    usuControlRemoto + ' > C:\resultado_query.txt');

      numProceso := ejecutarEsperar(obtenerDirectorioSistema +  '\cmd.exe', '/c query user ' +
          usuControlRemoto + ' > C:\resultado_query.txt');
      if numProceso <> 0 then
        WaitForSingleObject(numProceso, INFINITE);

      if FileExists('C:\resultado_query.txt') then
      begin
        listaFichero := TStringList.Create;
        listaFichero.LoadFromFile('C:\resultado_query.txt');
        if listaFichero.Count > 1 then
        begin
          if MessageDlg('Desea realizar control remoto al usuario [' +
              usuControlRemoto + ']?', mtconfirmation,
              [mbyes, mbno], 0) = mryes then
          begin
            idSesion := trim (copy (listaFichero.Strings[1], 23, 19));
            ejecutarFichero ('cmd.exe', ' /c shadow ' + idSesion);
          end;
        end
        else
          MessageDlg('El usuario [' + usuControlRemoto +
              '] no tiene sesin abierta en el servidor [' +
              nombreServidor + '].', mtInformation, [mbok], 0);
        DeleteFile(pchar('C:\resultado_query.txt'));
      end;
    end;
    if iniciarSTS then
      ejecutarFichero ('tsadmin.exe', '');
    //*******FIN Iniciar Administrador de servicios de Terminal Server
    //y hacer control remoto si as se ha indicado

  except
  end;
end.
